// UMLDecorator.cpp : Implementation of CUMLDecorator
#include "stdafx.h"
#include "Decorator.h"


#define VERIFY_INIT   { if (!m_isInitialized) return E_DECORATOR_UNINITIALIZED; }
#define VERIFY_LOCSET { if (!m_isLocSet) return E_DECORATOR_LOCISNOTSET; }

CDecoratorUtil	d_util;

#define SAMPLE_SIZEX	40
#define SAMPLE_SIZEY	20

/////////////////////////////////////////////////////////////////////////////
// CDecorator
STDMETHODIMP CDecorator::Initialize(IMgaProject *project, IMgaMetaPart *metaPart, IMgaFCO *obj)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState( ));
	//
	// TODO: read all important data from MGA and cache them for later use
	//
	m_mgaFco = obj;		// obj == NULL, if we are in the PartBrowser
	
	if (!GetMetaFCO(metaPart, m_metaFco)) {
		return E_DECORATOR_INIT_WITH_NULL;
	}
	else {
		m_isInitialized = true;
	}	
	
	if (m_mgaFco) {
		CComBSTR bstr;
		COMTHROW(m_mgaFco->get_Name(&bstr));
		m_name = bstr;
	}
	else {
		CComBSTR bstr;
		COMTHROW(m_metaFco->get_DisplayedName(&bstr));
		if (bstr.Length() == 0 ) {
			bstr.Empty();
			COMTHROW(m_metaFco->get_Name(&bstr));
		}
		m_name = bstr;
	}

	if(!GetColorPreference(m_color,COLOR_PREF)) {
		m_color = GME_BLACK_COLOR;
	}
	if(!GetColorPreference(m_nameColor,NAME_COLOR_PREF)) {
		m_nameColor = GME_BLACK_COLOR;
	}

	if (m_mgaFco)
	{
		CComPtr<IMgaFCO> mgaFco = m_mgaFco;
		CComBSTR type("slevel");
		long intVal;

		try {
			COMTHROW(mgaFco->get_IntAttrByName(type, &intVal));
		}
		catch (hresult_exception &ee) {
			AfxMessageBox(ee.what());
		}

		m_SatisfactionLevel = (SatisfactionLevel)intVal;
		m_IconSatisfied =::LoadIcon(AfxGetResourceHandle(), MAKEINTRESOURCE(IDI_ICON_SATISFIED));
		m_IconWeaklySatisfied = ::LoadIcon(AfxGetResourceHandle(), MAKEINTRESOURCE(IDI_ICON_WEAKLY_SATISFIED));
		m_IconUndecided = ::LoadIcon(AfxGetResourceHandle(), MAKEINTRESOURCE(IDI_ICON_UNDECIDED));
		m_IconWeaklyDenied = ::LoadIcon(AfxGetResourceHandle(), MAKEINTRESOURCE(IDI_ICON_WEAKLY_DENIED));
		m_IconDenied = ::LoadIcon(AfxGetResourceHandle(), MAKEINTRESOURCE(IDI_ICON_DENIED));
		m_IconConflict = ::LoadIcon(AfxGetResourceHandle(), MAKEINTRESOURCE(IDI_ICON_CONFLICT));
	}

	CalcShapeAndPositions();
	return S_OK;
}

STDMETHODIMP CDecorator::Destroy()
{
	//
	// TODO: At least free all references to MGA objects
	//
	VERIFY_INIT;
	m_isInitialized = false;
	m_isLocSet = false;
	m_metaFco = NULL; m_mgaFco = NULL;
	return S_OK;
}

STDMETHODIMP CDecorator::GetMnemonic(BSTR *mnemonic)
{	
	//
	// TODO: Return the logical name of the decorator (currently not used by GME)
	//
	*mnemonic = CComBSTR(DECORATOR_NAME).Detach();
	return S_OK;
}

STDMETHODIMP CDecorator::GetFeatures(feature_code *features)
{	
	//
	// TODO: Return supported features (combine multiple features with bitwise-OR)
	// Available feature codes are found in MgaDecorator.idl
	// (curently not used by GME)
	*features = 0;
	return S_OK;
}

STDMETHODIMP CDecorator::SetParam(BSTR name, VARIANT value)
{
	//
	// TODO:  Parse and set all supported parameters, otherwise return error
	// (currently all values are BSTR type)
	//
	VERIFY_INIT;
	return E_DECORATOR_UNKNOWN_PARAMETER;
}

STDMETHODIMP CDecorator::GetParam(BSTR name, VARIANT* value)
{
	//
	// TODO: Return values of supported and previously set parameters, otherwise return error
	// (currently GME does not use this method)
	//
	VERIFY_INIT;
	return E_DECORATOR_UNKNOWN_PARAMETER;
}

STDMETHODIMP CDecorator::SetActive(VARIANT_BOOL isActive)
{
	//
	// TODO: If isActive==VARIANT_FALSE, draw your object in GME_GREYED_OUT, otherwise use the color of the object
	//
	VERIFY_INIT;
	m_isActive = (isActive != VARIANT_FALSE);
	return S_OK;
}

STDMETHODIMP CDecorator::GetPreferredSize(long* sizex, long* sizey) 
{
	//
	// TODO: Give GME a hint about the object size. Do not expect GME to take it into account
	//
	VERIFY_INIT;
	*sizex = m_ShapeWidth;
	*sizey = m_ShapeHeight;
	return S_OK;
}

void CDecorator::CalcShapeAndPositions(CDC *pDC)
{
	int maxWidth = 0;
	int shape_height;
	CSize ext;
	CDC	dc;
	CDisplayArray display_strings;
	int i;
	
	dc.Attach(pDC ? pDC->m_hDC : GetDC(NULL));			// Trick
	CFont *oldfont = dc.SelectObject(d_util.GetFont(GME_PORTNAME_FONT));
	
	ParseDisplayStrings(m_name, &display_strings);

	shape_height = 0;
	for (i = 0; i < display_strings.GetCount(); i++)
	{
		ext = dc.GetTextExtent(display_strings.GetAt(i));
		maxWidth = max(maxWidth, ext.cx);
		shape_height += ext.cy;
	}

	m_ShapeWidth = maxWidth + 36;
	m_ShapeHeight = shape_height + 19;

	if (pDC) {
		dc.SelectObject(oldfont);
		dc.Detach();
	}
}

void CDecorator::ParseDisplayStrings(CString name, CDisplayArray *display_strings)
{
	const int MAX_CHAR_PERLINE = 21;
	CString str;
	int next_space_index;
	bool reached_end;

	reached_end = false;
	while (!reached_end)
	{
		str = name.Mid(0, MAX_CHAR_PERLINE);

		if (str.GetLength() == name.GetLength())
		{
			// End is reached.
			display_strings->Add(str);
			reached_end = true;
		}
		else
		{
			next_space_index = str.ReverseFind(' ');

			if (next_space_index >= 0)
			{
				// Found one space character and there is more to parse.
				str = name.Mid(0, next_space_index);
				display_strings->Add(str);

				name = name.Mid(next_space_index + 1);
			}
			else
			{
				next_space_index = MAX_CHAR_PERLINE - 1;
				str = name.Mid(0, MAX_CHAR_PERLINE);
				display_strings->Add(str);

				name = name.Mid(next_space_index + 1);
			}
		}
	}
}


STDMETHODIMP CDecorator::SetLocation(long sx, long sy, long ex, long ey)
{
	//
	// TODO: Draw the object exactly to the this location later
	//
	VERIFY_INIT;
	m_sx = sx;
	m_sy = sy;
	m_ex = ex;
	m_ey = ey;
	m_isLocSet = true;
	return S_OK;
}

STDMETHODIMP CDecorator::GetLocation(long *sx, long *sy, long *ex, long *ey)
{
	//
	// TODO: Return previously set location parameters
	// (currently GME does not call this)
	// 
	VERIFY_INIT;
	VERIFY_LOCSET;
	*sx = m_sx;
	*sy = m_sy;
	*ex = m_ex;
	*ey = m_ey;
	return S_OK;
}

STDMETHODIMP CDecorator::GetLabelLocation(long *sx, long *sy, long *ex, long *ey)
{
	//
	// TODO: Return the location of the text box of your label if you support labels.
	// (currently GME does not call this)
	//
	VERIFY_INIT;
	VERIFY_LOCSET;
	return S_OK;
}

STDMETHODIMP CDecorator::GetPortLocation(IMgaFCO *fco, long *sx, long *sy, long *ex, long *ey)
{
	//
	// TODO: Return the location of the specified port if ports are supported in the decorator
	//
	VERIFY_INIT;
	VERIFY_LOCSET;

	return E_DECORATOR_PORTNOTFOUND;
}

STDMETHODIMP CDecorator::GetPorts(IMgaFCOs **portFCOs)
{
	//
	// TODO: Return a collection of mga objects represented as ports.
	//
	VERIFY_INIT;
	CComPtr<IMgaFCOs> coll;
	COMTHROW(coll.CoCreateInstance(OLESTR("Mga.MgaFCOs")));
	*portFCOs = coll.Detach();
	return S_OK;
}


STDMETHODIMP CDecorator::Draw(HDC hdc)
{
	//
	// TODO: Draw your object. Saving & restoring the HDC is a good practice, however it is not a strict requirement
	// Use d_util for brushes/pens/fonts.
	//
	VERIFY_INIT;
	VERIFY_LOCSET;
	CDC dc;
	CPen *pen;
	dc.Attach(hdc);

	if (dc.IsPrinting()) 
	{
		CalcShapeAndPositions(&dc);
	}

	CPoint p1(m_sx + m_ShapeWidth, m_sy + m_ShapeHeight - 16);
	CRect rect(m_sx, m_sy, m_sx + m_ShapeWidth, m_sy + m_ShapeHeight);

	// Draw the icon for satisfaction level
	if (m_SatisfactionLevel == LevelSatisfied)
	{
		dc.DrawIcon(p1, m_IconSatisfied);
	}
	else if (m_SatisfactionLevel == LevelWeaklySatisfied)
	{
		dc.DrawIcon(p1, m_IconWeaklySatisfied);
	}
	else if (m_SatisfactionLevel == LevelUndecided)
	{
		dc.DrawIcon(p1, m_IconUndecided);
	}
	else if (m_SatisfactionLevel == LevelWeaklyDenied)
	{
		dc.DrawIcon(p1, m_IconWeaklyDenied);
	}
	else if (m_SatisfactionLevel == LevelDenied)
	{
		dc.DrawIcon(p1, m_IconDenied);
	}
	else if (m_SatisfactionLevel == LevelConflict)
	{
		dc.DrawIcon(p1, m_IconConflict);
	}

	// Draw the shape of this element
	pen = new CPen();
	pen->CreatePen(PS_SOLID, 2, m_nameColor);
	CPen *oldpen = dc.SelectObject(pen);

	CPoint pts[6];

	pts[0].x = rect.left +5;
	pts[0].y = rect.top;

	pts[1].x = rect.left + rect.Width() - 5;
	pts[1].y = pts[0].y;

	pts[2].x = rect.left + rect.Width();
	pts[2].y = rect.top + rect.Height() / 2;

	pts[3].x = pts[1].x;
	pts[3].y = rect.bottom;

	pts[4].x = pts[0].x;
	pts[4].y = pts[3].y;

	pts[5].x = rect.left;
	pts[5].y = pts[2].y;

	dc.Polygon(pts, 6);


	dc.SelectObject(oldpen);

	// Draw the text in the element shape
	CDisplayArray display_strings;
	ParseDisplayStrings(m_name, &display_strings);

	CSize ext;
	int i;
	CPoint text_point;
	CString str;

	text_point.x = (m_sx + m_ex) / 2;
	text_point.y = m_sy + 26;
	for (i = 0; i < display_strings.GetCount(); i++)
	{
		str = display_strings.GetAt(i);
		ext = dc.GetTextExtent(str);
		d_util.DrawText(&dc, str, text_point, d_util.GetFont(GME_NAME_FONT), m_nameColor, TA_BOTTOM | TA_CENTER);
		text_point.y += ext.cy;
	}

	dc.Detach();

	return S_OK;
}

STDMETHODIMP CDecorator::SaveState()
{
	//
	// TODO: The only method where we are in read-write transaction. Store all permanent information
	// (currently GME does not support this) 
	//
	VERIFY_INIT;
	return S_OK;
}


//////////// Decorator private functions
CDecorator::CDecorator() : m_sx(0), m_sy(0), m_ex(0), m_ey(0), 
	m_isActive(true), m_mgaFco(0), m_metaFco(0),
	m_isInitialized(false), m_isLocSet(false),  
	m_color(GME_BLACK_COLOR), m_nameColor(GME_BLACK_COLOR)
{
}

CDecorator::~CDecorator()
{
}

bool CDecorator::GetMetaFCO(const CComPtr<IMgaMetaPart> &metaPart, CComPtr<IMgaMetaFCO> &metaFco)
{
	if (!metaPart) {
		return false;
	}

	metaFco = NULL;
	CComPtr<IMgaMetaRole> metaRole;
	try {
		COMTHROW(metaPart->get_Role(&metaRole));
		COMTHROW(metaRole->get_Kind(&metaFco));
	}
	catch (hresult_exception &) {
		metaFco = NULL;
	}
	return (metaFco != NULL);
}

bool CDecorator::GetPreference(CString &val,const CString &path)
{
	CComBSTR pathBstr(path);
	CComBSTR bstrVal;
	if (m_mgaFco) {
		COMTHROW(m_mgaFco->get_RegistryValue(pathBstr,&bstrVal));
	}
	else {
		COMTHROW(m_metaFco->get_RegistryValue(pathBstr,&bstrVal));
	}
	val = bstrVal;
	return !val.IsEmpty();
}

bool CDecorator::GetPreference(int &val,const CString &path,bool hex)
{
	CString strVal;
	GetPreference(strVal, path);
	return (sscanf(strVal,hex ? "%x" : "%d",&val) == 1);
}

bool CDecorator::GetColorPreference(unsigned long &color, const CString &path)
{
	int i;
	if(GetPreference(i,path,true)) {
		unsigned int r = (i & 0xff0000) >> 16;
		unsigned int g = (i & 0xff00) >> 8;
		unsigned int b = i & 0xff;
		color = RGB(r,g,b);
		return true;
	}
	return false;
}